"use strict";
const vue = require("vscode-vue-languageservice");
const shared = require("@volar/shared");
const path = require("upath");
const init = (modules) => {
    const { typescript: ts } = modules;
    const vueFilesGetter = new WeakMap();
    const pluginModule = {
        create(info) {
            // fix: https://github.com/johnsoncodehk/volar/issues/205
            info.project.getScriptKind = fileName => {
                switch (path.extname(fileName)) {
                    case '.vue': return ts.ScriptKind.TSX; // can't use External, Unknown
                    case '.js': return ts.ScriptKind.JS;
                    case '.jsx': return ts.ScriptKind.JSX;
                    case '.ts': return ts.ScriptKind.TS;
                    case '.tsx': return ts.ScriptKind.TSX;
                    case '.json': return ts.ScriptKind.JSON;
                    default: return ts.ScriptKind.Unknown;
                }
            };
            const proxyHost = createProxyHost(ts, info);
            const vueLs = vue.createLanguageService(modules, proxyHost.host, true);
            vueFilesGetter.set(info.project, proxyHost.getVueFiles);
            return new Proxy(info.languageService, {
                get: (target, property) => {
                    return vueLs.__internal__.tsPlugin[property] || target[property];
                },
            });
        },
        getExternalFiles(project) {
            const getVueFiles = vueFilesGetter.get(project);
            if (!getVueFiles) {
                return [];
            }
            return getVueFiles().filter(fileName => project.fileExists(fileName));
        },
    };
    return pluginModule;
};
function createProxyHost(ts, info) {
    let projectVersion = 0;
    let anyFilesChanged = false;
    let disposed = false;
    let vueFiles = new Set(getVueFiles());
    const fileWatchers = new Map();
    const scriptVersions = new Map();
    const scriptSnapshots = new Map();
    const host = {
        getNewLine: () => ts.sys.newLine,
        useCaseSensitiveFileNames: () => ts.sys.useCaseSensitiveFileNames,
        readFile: ts.sys.readFile,
        writeFile: ts.sys.writeFile,
        fileExists: ts.sys.fileExists,
        directoryExists: ts.sys.directoryExists,
        getDirectories: ts.sys.getDirectories,
        readDirectory: ts.sys.readDirectory,
        realpath: ts.sys.realpath,
        getCompilationSettings: () => info.project.getCompilationSettings(),
        getCurrentDirectory: () => info.project.getCurrentDirectory(),
        getDefaultLibFileName: () => info.project.getDefaultLibFileName(),
        getProjectVersion,
        getScriptFileNames,
        getScriptVersion,
        getScriptSnapshot,
    };
    checkFilesAddRemove();
    const directoryWatcher = info.serverHost.watchDirectory(info.project.getCurrentDirectory(), onFileChangedOrConfigUpdated, true);
    let tsconfigWatcher = info.project.fileExists(info.project.projectName)
        ? info.serverHost.watchFile(info.project.projectName, () => {
            onFileChangedOrConfigUpdated();
            onProjectUpdated();
        })
        : undefined;
    return {
        host,
        getVueFiles: () => [...vueFiles],
        dispose,
    };
    async function onFileChangedOrConfigUpdated() {
        anyFilesChanged = true;
        await shared.sleep(0);
        if (anyFilesChanged && !disposed) {
            anyFilesChanged = false;
            vueFiles = new Set(getVueFiles());
            checkFilesAddRemove();
        }
    }
    function getProjectVersion() {
        return info.project.getProjectVersion() + ':' + projectVersion.toString();
    }
    function getScriptFileNames() {
        return info.project.getScriptFileNames().concat([...vueFiles]);
    }
    function getScriptVersion(fileName) {
        var _a;
        if (vueFiles.has(fileName)) {
            return ((_a = scriptVersions.get(fileName)) !== null && _a !== void 0 ? _a : -1).toString();
        }
        return info.project.getScriptVersion(fileName);
    }
    function getScriptSnapshot(fileName) {
        if (vueFiles.has(fileName)) {
            const version = getScriptVersion(fileName);
            const cache = scriptSnapshots.get(fileName);
            if (cache && cache[0] === version) {
                return cache[1];
            }
            const text = getScriptText(fileName);
            if (text !== undefined) {
                const snapshot = ts.ScriptSnapshot.fromString(text);
                scriptSnapshots.set(fileName, [version.toString(), snapshot]);
                return snapshot;
            }
            return;
        }
        return info.project.getScriptSnapshot(fileName);
    }
    function getScriptText(fileName) {
        if (info.project.fileExists(fileName)) {
            return info.project.readFile(fileName);
        }
    }
    function getVueFiles() {
        const parseConfigHost = {
            useCaseSensitiveFileNames: info.project.useCaseSensitiveFileNames(),
            readDirectory: (path, extensions, exclude, include, depth) => {
                return info.project.readDirectory(path, ['.vue'], exclude, include, depth);
            },
            fileExists: fileName => info.project.fileExists(fileName),
            readFile: fileName => info.project.readFile(fileName),
        };
        const { fileNames } = ts.parseJsonConfigFileContent({}, parseConfigHost, info.project.getCurrentDirectory(), info.project.getCompilerOptions());
        return fileNames;
    }
    function checkFilesAddRemove() {
        let changed = false;
        for (const fileName of fileWatchers.keys()) {
            if (!vueFiles.has(fileName)) {
                fileWatchers.get(fileName).close();
                fileWatchers.delete(fileName);
                changed = true;
            }
        }
        for (const fileName of vueFiles) {
            if (!fileWatchers.has(fileName)) {
                const fileWatcher = info.serverHost.watchFile(fileName, (fileName, eventKind) => {
                    if (eventKind === ts.FileWatcherEventKind.Changed) {
                        onFileChanged(fileName);
                    }
                });
                fileWatchers.set(fileName, fileWatcher);
                changed = true;
            }
        }
        if (changed) {
            onProjectUpdated();
        }
    }
    function onFileChanged(fileName) {
        fileName = path.resolve(fileName);
        const oldVersion = scriptVersions.get(fileName);
        if (oldVersion === undefined) {
            scriptVersions.set(fileName, 1);
        }
        else {
            scriptVersions.set(fileName, oldVersion + 1);
        }
        onProjectUpdated();
    }
    function onProjectUpdated() {
        projectVersion++;
        info.project.refreshDiagnostics();
    }
    function dispose() {
        directoryWatcher.close();
        if (tsconfigWatcher) {
            tsconfigWatcher.close();
        }
        for (const [_, fileWatcher] of fileWatchers) {
            fileWatcher.close();
        }
        disposed = true;
    }
}
module.exports = init;
//# sourceMappingURL=index.js.map